/******************************************************************************
  Copyright (c) 2008,2009 by Turku PET Centre

  scanditronics.h

  Modification history:
  2008-12-13 Vesa Oikonen
  2008-12-14 Vesa Oikonen
  2009-03-13 VO
             Added start_time to struct.
             Definitions for new functions.
  2009-03-16 VO
             sampler_id replaced by enum BSampler.
             format replaced by enum BFormat.
             Filename corrected to scanditronics.h.

******************************************************************************/
#ifndef _SCANDITRONICS_H
#define _SCANDITRONICS_H
/*****************************************************************************/

/*****************************************************************************/
#define MAX_SAMPLER_HEADERS 25
#define MAX_SAMPLER_LINE_LEN 1024
#define SAMPLER_GMT_ADD 2; /* hours added to Greenwich main time*/
/*****************************************************************************/
/** Individual ID number for online blood sampler, so that appropriate
 *  calibration, dead-time and dispersion correction coefficients can
 *  be applied. */
typedef enum {
  BSAMPLER_UNKNOWN,BSAMPLER_SCANDITRONICS,BSAMPLER_GEMS,BSAMPLER_ALLOGG
} BSampler;
/** File format for online blood sampler data */
typedef enum {
  BFORMAT_UNKNOWN,BFORMAT_SCANDITRONICS,BFORMAT_ALLOGG
} BFormat;

typedef struct {
  double time_of_day;
  double time_since_start;
  double meas_interval;
  int pair1_coincid;
  int pair1_det1_cnts;
  int pair1_det2_cnts;
  int pair2_coincid;
  int pair2_det1_cnts;
  int pair2_det2_cnts;
  int aux_cnts;
  int check;
  double t, ch1, ch2, ch_mean;
} BSample;

typedef struct {
  /** Individual ID number for online blood sampler, so that appropriate
   *  calibration, dead-time and dispersion correction coefficients can
   *  be applied. */
  BSampler sampler_id;
  /** File format for online blood sampler data */
  BFormat format;
  /** Nr of blood samples contained in this struct */
  int sample_nr;
  /** Space that is allocated for blood samples */
  int sample_space;
  /** Nr of header lines */
  int header_nr;
  /** Pointer to list of sample data */
  BSample *sample;
  /** List of headers, including any starting character like '#' */
  char header[MAX_SAMPLER_HEADERS][MAX_SAMPLER_LINE_LEN];
  /** Sampler start date and time (time of first sample) */
  time_t start_time;
} BData;
/*****************************************************************************/
extern void bdata_init(BData *data);
extern void bdata_empty(BData *data);
extern int bdata_allocate(BData *data, int size);
extern int bdata_read(
  char *filename, BData *bdata, int verbose, char *status);
extern int bdata_calc(BData *bdata);
extern int bdata_examine(
  BData *bdata, double limit, double *r, double *ch1, double *ch2);
extern int bdata_get_date(
  BData *bdata, int *yyyy, int *mm, int *dd, int *h, int *m, int *s);
extern int bdata_coinc_count(
  BData *bdata, int *ch1ok, int *ch2ok, int *chbok);
extern int bdata_write(
  BData *bdata, char *filename, int verbose, char *status);
extern int bdata_get_start_time(BData *bdata, int verbose, char *status);
extern int bdata_correct_header(BData *bdata);
/*****************************************************************************/
#endif
