/** @file rndpoint.c
 *  @brief Random point for optimization routines.
 */
/*****************************************************************************/
#include "tpcclibConfig.h"
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
/*****************************************************************************/
#include "tpcextensions.h"
#include "tpcrand.h"
/*****************************************************************************/
#include "tpcnlopt.h"
/*****************************************************************************/

/*****************************************************************************/
/** Create random parameters between specified limits. 
    @pre Uses rand(), therefore set seed for a new series of pseudo-random 
    numbers; to produce truly random numbers (not just pseudo-random), 
    do srand(time(0)) before calling this function. If no seed is set, then 
    value 1 is used as default seed by rand().
    @sa drand
    @author Vesa Oikonen
 */
int nloptRandomPoint(
  /** Pointer to parameter list to be filled */
  double *p,
  /** Pointer to parameter lower limits */
  double *low,
  /** Pointer to parameter upper limits */
  double *up,
  /** List length */
  unsigned int n
) {
  unsigned int i;
  double dif;

  if(p==NULL || low==NULL || up==NULL) return(1);
  if(n==0) return(0);

  for(i=0; i<n; i++) {
    dif=up[i]-low[i];
    if(dif<=0.0) p[i]=low[i]; else p[i]=low[i]+drand()*dif;
  }

  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
