/*****************************************************************************/
#include "tpcclibConfig.h"
/*****************************************************************************/
#include "tpcextensions.h"
#include "test_tpcnlopt.h"
/*****************************************************************************/

/*****************************************************************************/
int test_nloptInit(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  NLOPT nlo;
  nloptInit((NLOPT*)NULL);
  if(verbose>1) printf("did not segfault, that is good.\n");
  nloptInit(&nlo);
  if(verbose>1) printf("did not segfault, that is good.\n");

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_nloptFree(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  nloptFree((NLOPT*)NULL);
  if(verbose>1) printf("did not segfault, that is good.\n");

  NLOPT nlo; nloptInit(&nlo);
  nloptFree(&nlo);
  if(verbose>1) printf("did not segfault, that is good.\n");

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_nloptAllocate(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  int ret;

  ret=nloptAllocate((NLOPT*)NULL, 1000);
  if(ret==TPCERROR_OK) return 1;
  if(verbose>1) printf("did not segfault, that is good.\n");

  NLOPT nlo; nloptInit(&nlo);
  ret=nloptAllocate(&nlo, 10);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return 2;}
  nloptFree(&nlo);
  if(verbose>1) printf("did not segfault, that is good.\n");

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_nloptDuplicate(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  int ret;

  ret=nloptDuplicate((NLOPT*)NULL, NULL);
  if(ret==TPCERROR_OK) return 1;
  if(verbose>1) printf("did not segfault, that is good.\n");

  NLOPT nlo1; nloptInit(&nlo1);
  ret=nloptDuplicate(&nlo1, NULL);
  if(ret==TPCERROR_OK) return 2;
  if(verbose>1) printf("did not segfault, that is good.\n");

  NLOPT nlo2; nloptInit(&nlo2);
  ret=nloptDuplicate(&nlo1, &nlo2);
  if(ret!=TPCERROR_OK) return 3;
  if(verbose>2) printf("did not segfault, that is good.\n");

  ret=nloptAllocate(&nlo1, 10);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo1); return 10;}
  ret=nloptDuplicate(&nlo1, &nlo2);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo1); nloptFree(&nlo2); return 11;}
  if(nlo2.totalNr!=10) {nloptFree(&nlo1); nloptFree(&nlo2); return 12;}

  nloptFree(&nlo1); nloptFree(&nlo2);
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_nloptLimitFixedNr(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  NLOPT nlo; nloptInit(&nlo);
  unsigned int i, n;

  if(verbose>1) printf("\nTesting with NULL input\n");
  n=nloptLimitFixedNr(NULL);
  if(n>0) return(1);
  n=nloptLimitFixedNr(&nlo);
  if(n>0) return(2);

  if(verbose>1) printf("\nTesting with reasonable input\n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(nloptAllocate(&nlo, 9)!=TPCERROR_OK) return(10);
  for(i=0; i<nlo.totalNr; i++) {
    nlo.xlower[i]=0.0; nlo.xupper[i]=1.0+i;
  }

  n=nloptLimitFixedNr(&nlo);
  if(n>0) {nloptFree(&nlo); return(11);}

  nlo.xupper[2]=nlo.xlower[2]; if(verbose>5) nloptWrite(&nlo, stdout);
  n=nloptLimitFixedNr(&nlo);
  if(n!=(unsigned int)1) {nloptFree(&nlo); return(12);}

  if(verbose>1) printf("\nTesting with NaNs\n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  nlo.xupper[3]=nlo.xlower[3]=nan("");
  n=nloptLimitFixedNr(&nlo);
  if(n!=(unsigned int)1) {nloptFree(&nlo); return(22);}

  nloptFree(&nlo);
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_nloptFixedNr(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  NLOPT nlo; nloptInit(&nlo);
  unsigned int i, n;

  if(verbose>1) printf("\nTesting with NULL input\n");
  n=nloptFixedNr(NULL);
  if(n>0) return(1);
  n=nloptFixedNr(&nlo);
  if(n>0) return(2);

  if(verbose>1) printf("\nTesting with reasonable input\n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(nloptAllocate(&nlo, 9)!=TPCERROR_OK) return(10);
  for(i=0; i<nlo.totalNr; i++) {
    nlo.xlower[i]=0.0; nlo.xupper[i]=1.0+i;
    nlo.xdelta[i]=0.0987*(nlo.xupper[i]-nlo.xlower[i]);
  }

  n=nloptFixedNr(&nlo);
  if(n>0) {nloptFree(&nlo); return(11);}

  nlo.xdelta[1]=0.0; if(verbose>5) nloptWrite(&nlo, stdout);
  n=nloptFixedNr(&nlo);
  if(n!=(unsigned int)1) {nloptFree(&nlo); return(12);}
  nlo.xupper[2]=nlo.xlower[2]; if(verbose>5) nloptWrite(&nlo, stdout);
  n=nloptFixedNr(&nlo);
  if(n!=(unsigned int)2) {nloptFree(&nlo); return(13);}
  nlo.xdelta[2]=0.0; if(verbose>5) nloptWrite(&nlo, stdout);
  n=nloptFixedNr(&nlo);
  if(n!=(unsigned int)2) {nloptFree(&nlo); return(14);}

  if(verbose>1) printf("\nTesting with NaNs\n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  nlo.xdelta[1]=nan(""); if(verbose>5) nloptWrite(&nlo, stdout);
  n=nloptFixedNr(&nlo);
  if(n!=(unsigned int)1) {nloptFree(&nlo); return(21);}
  nlo.xupper[2]=nlo.xlower[2]=nan("");
  n=nloptFixedNr(&nlo);
  if(n!=(unsigned int)1) {nloptFree(&nlo); return(22);}
  nlo.xdelta[2]=nan(""); if(verbose>5) nloptWrite(&nlo, stdout);
  n=nloptFixedNr(&nlo);
  if(n!=(unsigned int)0) {nloptFree(&nlo); return(23);}

  nloptFree(&nlo);
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_nloptRemoveEmpties(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  NLOPT nlo; nloptInit(&nlo);
  unsigned int i;

  if(verbose>1) printf("\nTesting with NULL input\n");
  nloptRemoveEmpties(NULL);
  if(verbose>1) printf("  -> good, did not crash\n");
  if(verbose>1) printf("\nTesting with empty input\n");
  nloptRemoveEmpties(&nlo);
  if(verbose>1) printf("  -> good, did not crash\n");

  if(verbose>1) printf("\nTesting with reasonable input\n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(nloptAllocate(&nlo, 9)!=TPCERROR_OK) return(10);
  for(i=0; i<nlo.totalNr; i++) {
    nlo.xlower[i]=0.0; nlo.xupper[i]=1.0+i;
    nlo.xfull[i]=0.45*(nlo.xupper[i]+nlo.xlower[i]);
    nlo.xdelta[i]=0.0987*(nlo.xupper[i]-nlo.xlower[i]);
  }

  if(verbose>5) nloptWrite(&nlo, stdout);
  nloptRemoveEmpties(&nlo);
  if(nlo.totalNr!=9) {nloptFree(&nlo); return(11);}

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  nlo.xfull[0]=nan("");
  if(verbose>5) nloptWrite(&nlo, stdout);
  nloptRemoveEmpties(&nlo);
  if(nlo.totalNr!=8) {nloptFree(&nlo); return(12);}

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  nlo.xdelta[0]=nan("");
  if(verbose>5) nloptWrite(&nlo, stdout);
  nloptRemoveEmpties(&nlo);
  if(nlo.totalNr!=7) {nloptFree(&nlo); return(13);}

  nloptFree(&nlo);
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
/** De Jong's second function or Rosenbrock's valley or Banana function.
 *  One minimum f(xi)=0. With n=2 minimum is at (1,1) and with n=3 at(1,1,1).
 *  With 4<=n<=7 there is one global minimum at (1,1,...,1) and local
 *  minimum somewhere near (-1,1,...1). For larger n the method breaks down.
 *  If used to test local optimization, note that initial guess must be quite
 *  close to the minimum.
 */
double nloptDejong2(
  /** Number of parameters. */
  int n, 
  /** Pointer to the parameter array of length n. */
  double *x, 
  /** Pointer to additional data to the function; not used here. */
  void *func_data
) {
  if(func_data==NULL) {/* just to prevent compiler warning */}
  double d1, d2, f=0.0;
  for(int i=0; i<n-1; i++) {
    d1=x[i+1]-x[i]*x[i];
    d2=1.0-x[i];
    f+=100.0*d1*d1 + d2*d2; 
  }
#if(0)
  printf("  p[]: %g", x[0]);
  for(int i=1; i<n; i++) printf(", %g", x[i]);
  printf(" => %g\n", f);
#endif
  return f;
}
/*****************************************************************************/

/*****************************************************************************/
/** Generalized Rastrigin function,
 *  f(x) = A*n + SUM(i=1..n; (xi^2 - A*cos(2*PI*xi))), where A=10 and xi=[-5.12,5.12].
 *  It has global minimum value f(x)=0.0 at xi=0.0, and large nr of local minima around it.
 *  If used to test local optimization, note that initial guess must be quite
 *  close to the minimum.
 */
double nloptRastrigin(
  /** Number of parameters. */
  int n,
  /** Pointer to the parameter array of length n. */
  double *x, 
  /** Pointer to additional data to the function; not used here. */
  void *func_data
) {
  if(func_data==NULL) {} // just to prevent compiler warning
  if(n<1 || n>50) return(nan(""));
  for(int i=0; i<n; i++) if(x[i]<-5.12 || x[i]>5.12) return(nan(""));

  double f=0.0;
  for(int i=0; i<n; i++)
    f+=fma(x[i], x[i], -10.0*cos(2.0*M_PI*x[i]));
  f+=10.0*n;
  return f;
}
/*****************************************************************************/

/*****************************************************************************/
/** Generalized Rastrigin function,
 *  modified so that 1.0 is added to the returned value.
 */
double nloptRastrigin1(
  /** Number of parameters. */
  int n,
  /** Pointer to the parameter array of length n. */
  double *x, 
  /** Pointer to additional data to the function; not used here. */
  void *func_data
) {
  if(func_data==NULL) {} // just to prevent compiler warning
  if(n<1 || n>50) return(nan(""));
  for(int i=0; i<n; i++) if(x[i]<-5.12 || x[i]>5.12) return(nan(""));

  double f=0.0;
  for(int i=0; i<n; i++)
    f+=fma(x[i], x[i], -10.0*cos(2.0*M_PI*x[i]));
  f+=10.0*n + 1.0;
  return f;
}
/*****************************************************************************/

/*****************************************************************************/
