/*****************************************************************************/
#include "tpcclibConfig.h"
/*****************************************************************************/
#include "tpcextensions.h"
#include "test_tpcnlopt.h"
/*****************************************************************************/

/*****************************************************************************/
int test_nloptITGO1(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  int i, ret=0, parNr=0;
  NLOPT nlo; nloptInit(&nlo);

  if(verbose>1) printf("\n calling function with empty input\n");
  ret=nloptITGO1(&nlo, 0, 0, 0, 0, NULL);
  if(ret==TPCERROR_OK) return(1);
  if(verbose>2) printf("  -> error returned, which is good\n");


  if(verbose>1) printf("\n Rosenbrock's valley with N=2 \n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  parNr=2;
  ret=nloptAllocate(&nlo, parNr);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(11);}
  /* settings */
  i=0; nlo.xfull[i]=0.9; nlo.xtol[i]=0.0001; nlo.xlower[i]=-100.0; nlo.xupper[i]=100.0;
  i=1; nlo.xfull[i]=1.1; nlo.xtol[i]=0.0001; nlo.xlower[i]=0.0; nlo.xupper[i]=20.0;
  /* set object function */
  nlo._fun=nloptDejong2;
  /* Optimize */
  ret=nloptITGO1(&nlo, 1, 0, 0, 0, status);
  if(verbose>2) printf("  -> ret := %d\n", ret);
  if(verbose>3) printf("  -> function_calls := %d\n", nlo.funCalls);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(12);}
  if(verbose>2)
    for(i=0; i<parNr; i++) printf("  x[%d] := %g\n", i, nlo.xfull[i]);
  /* Check optimized parameters */
  ret=0;
  for(int i=0; i<parNr; i++) {
    double e=fabs(nlo.xfull[i]-1.0);
    if(e>1.0E-03) {
      ret++; if(verbose>2) printf("  |x-true|=%g\n", e);
    }
  }
  if(ret>0) {
    if(verbose>0) printf("%s did not converge to correct minimum.\n", __func__);
    fflush(stdout);
  }
  nloptFree(&nlo);


  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_nloptITGO2(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  int i, ret=0, parNr=0;
  NLOPT nlo; nloptInit(&nlo);

  if(verbose>1) printf("\n calling function with empty input\n");
  ret=nloptITGO2(&nlo, 0, 0, 0, 0, NULL);
  if(ret==TPCERROR_OK) return(1);
  if(verbose>2) printf("  -> error returned, which is good\n");


  if(verbose>1) printf("\n Rosenbrock's valley with N=2 \n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  parNr=2;
  ret=nloptAllocate(&nlo, parNr);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(11);}
  /* settings */
  i=0; nlo.xfull[i]=0.9; nlo.xtol[i]=0.0001; nlo.xlower[i]=-100.0; nlo.xupper[i]=100.0;
  i=1; nlo.xfull[i]=1.1; nlo.xtol[i]=0.0001; nlo.xlower[i]=0.0; nlo.xupper[i]=20.0;
  /* set object function */
  nlo._fun=nloptDejong2;
  /* Optimize */
  ret=nloptITGO2(&nlo, 1, 0, 0, 0, status);
  if(verbose>2) printf("  -> ret := %d\n", ret);
  if(verbose>3) printf("  -> function_calls := %d\n", nlo.funCalls);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(12);}
  if(verbose>2)
    for(i=0; i<parNr; i++) printf("  x[%d] := %g\n", i, nlo.xfull[i]);
  /* Check optimized parameters */
  ret=0;
  for(int i=0; i<parNr; i++) {
    double e=fabs(nlo.xfull[i]-1.0);
    if(e>1.0E-03) {
      ret++; if(verbose>2) printf("  |x-true|=%g\n", e);
    }
  }
  if(ret>0) {
    if(verbose>0) printf("%s did not converge to correct minimum.\n", __func__);
    fflush(stdout);
  }
  nloptFree(&nlo);


  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_nloptIATGO(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  int i, ret=0, parNr=0;
  NLOPT nlo; nloptInit(&nlo);

  if(verbose>1) printf("\n calling function with empty input\n");
  ret=nloptIATGO(&nlo, 0, 0, 0.5, NULL);
  if(ret==TPCERROR_OK) return(1);
  if(verbose>2) printf("  -> error returned, which is good\n");


  if(verbose>1) printf("\n Rosenbrock's valley with N=2 \n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  parNr=2;
  ret=nloptAllocate(&nlo, parNr);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(11);}
  /* settings */
  i=0; nlo.xfull[i]=0.9; nlo.xtol[i]=0.0001; nlo.xlower[i]=-100.0; nlo.xupper[i]=100.0;
  i=1; nlo.xfull[i]=1.1; nlo.xtol[i]=0.0001; nlo.xlower[i]=0.0; nlo.xupper[i]=20.0;
  nlo.maxFunCalls=1500;
  /* set object function */
  nlo._fun=nloptDejong2;
  /* Optimize */
  ret=nloptIATGO(&nlo, 1, 0, 0.5, status);
  if(verbose>2) printf("  -> ret := %d\n", ret);
  if(verbose>2) printf("  -> function_calls := %d\n", nlo.funCalls);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(12);}
  if(verbose>2) for(i=0; i<parNr; i++) printf("  x[%d] := %g\n", i, nlo.xfull[i]);
  if(verbose>2) printf("  R=%g\n", nlo.funval);
  /* Check optimized parameters */
  ret=0;
  for(int i=0; i<parNr; i++) {
    double e=fabs(nlo.xfull[i]-1.0);
    if(e>1.0E-03) {
      ret++; if(verbose>2) printf("  |x-true|=%g\n", e);
    }
  }
  if(ret>0) {
    if(verbose>0) printf("%s did not converge to correct minimum.\n", __func__);
    fflush(stdout);
  }
  nloptFree(&nlo);


  if(verbose>1) printf("\n Rosenbrock's valley with N=6 \n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  parNr=6;
  ret=nloptAllocate(&nlo, parNr);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(21);}
  /* settings */
  i=0; nlo.xfull[i]=0.91; nlo.xtol[i]=0.0001; nlo.xlower[i]=-100.0; nlo.xupper[i]=100.0;
  i++; nlo.xfull[i]=1.11; nlo.xtol[i]=0.0001; nlo.xlower[i]=-100.0; nlo.xupper[i]=10.0;
  i++; nlo.xfull[i]=0.95; nlo.xtol[i]=0.0001; nlo.xlower[i]=-10.0;  nlo.xupper[i]=20.0;
  i++; nlo.xfull[i]=1.05; nlo.xtol[i]=0.0001; nlo.xlower[i]=-100.0; nlo.xupper[i]=50.0;
  i++; nlo.xfull[i]=0.99; nlo.xtol[i]=0.0001; nlo.xlower[i]=-50.0;  nlo.xupper[i]=150.0;
  i++; nlo.xfull[i]=1.15; nlo.xtol[i]=0.0001; nlo.xlower[i]=-50.0;  nlo.xupper[i]=100.0;
  i++; nlo.maxFunCalls=10000;
  /* set object function */
  nlo._fun=nloptDejong2;
  /* Optimize */
  ret=nloptIATGO(&nlo, 1, 0, 0.15, status);
  if(verbose>2) printf("  -> ret := %d\n", ret);
  if(verbose>2) printf("  -> function_calls := %d\n", nlo.funCalls);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(22);}
  if(verbose>2) for(int i=0; i<parNr; i++) printf("  x[%d] := %g\n", i, nlo.xfull[i]);
  if(verbose>2) printf("  R=%g\n", nlo.funval);
  /* Check optimized parameters */
  ret=0;
  for(int i=0; i<parNr; i++) {
    double e=fabs(nlo.xfull[i]-1.0);
    if(e>5.0E-03) {
      ret++; if(verbose>2) printf("  |x-true|=%g\n", e);
    }
  }
  if(ret>0) {
    if(verbose>0) printf("%s did not converge to correct minimum.\n", __func__);
    fflush(stdout);
  }
  nloptFree(&nlo);


  if(verbose>1) printf("\n Generalized Rastrigin function with N=3 \n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  parNr=3;
  ret=nloptAllocate(&nlo, parNr);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(31);}
  /* settings */
  i=0; nlo.xfull[i]=0.11; nlo.xtol[i]=0.0001; nlo.xlower[i]=-4.0; nlo.xupper[i]=5.0;
  i++; nlo.xfull[i]=-0.1; nlo.xtol[i]=0.0001; nlo.xlower[i]=-5.0; nlo.xupper[i]=3.0;
  i++; nlo.xfull[i]=-0.2; nlo.xtol[i]=0.0001; nlo.xlower[i]=-3.0; nlo.xupper[i]=4.0;
  i++; nlo.maxFunCalls=100000;
  /* set object function */
  nlo._fun=nloptRastrigin1;
  /* Optimize */
  ret=nloptIATGO(&nlo, 1, 0, 0.10, status);
  if(verbose>2) printf("  -> ret := %d\n", ret);
  if(verbose>2) printf("  -> function_calls := %d\n", nlo.funCalls);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(32);}
  if(verbose>2) for(int i=0; i<parNr; i++) printf("  x[%d] := %g\n", i, nlo.xfull[i]);
  if(verbose>2) printf("  R=%g\n", nlo.funval);
  /* Check optimized parameters */
  ret=0;
  for(int i=0; i<parNr; i++) {
    double e=fabs(nlo.xfull[i]-0.0);
    if(e>5.0E-03) {
      ret++; if(verbose>2) printf("  |x-true|=%g\n", e);
    }
  }
  if(ret>0) {
    if(verbose>0) printf("%s did not converge to correct minimum.\n", __func__);
    fflush(stdout);
  }
  nloptFree(&nlo);



  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
