/*****************************************************************************/
#include "tpcclibConfig.h"
/*****************************************************************************/
#include "tpcextensions.h"
#include "test_tpctac.h"
/*****************************************************************************/

/*****************************************************************************/
int test_tacWCopy(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }
  
  int ret;
  TAC tac1, tac2; 
  tacInit(&tac1); tacInit(&tac2);

  if(verbose>1) printf("\n trying with empty data\n");
  ret=tacWCopy(&tac1, &tac2, 0, 0);
  if(ret==TPCERROR_OK) {tacFree(&tac1); tacFree(&tac2); return 1;}
  
  if(verbose>1) printf("\n creating test data\n");
  ret=create_tac(&tac1); if(ret==TPCERROR_OK) ret=create_tac(&tac2);  
  if(ret!=TPCERROR_OK) {tacFree(&tac1); tacFree(&tac2); return 10;}

  if(verbose>1) printf("\n checking indices\n");
  ret=tacWCopy(&tac1, &tac2, 2, 0);
  if(ret==TPCERROR_OK) {tacFree(&tac1); tacFree(&tac2); return 11;}
  ret=tacWCopy(&tac1, &tac2, -1, 1);
  if(ret==TPCERROR_OK) {tacFree(&tac1); tacFree(&tac2); return 12;}
  ret=tacWCopy(&tac1, &tac2, 0, tac1._sampleNr);
  if(ret==TPCERROR_OK) {tacFree(&tac1); tacFree(&tac2); return 13;}
  
  if(verbose>1) printf("\n checking copying\n");
  for(int i=0; i<tac2.sampleNr; i++) tac2.w[i]=nan("");
  tac2.weighting=WEIGHTING_OFF;
  for(int i=0; i<tac1.sampleNr; i++) tac1.w[i]=(double)(i+1);
  tac1.weighting=WEIGHTING_ON;
  
  ret=tacWCopy(&tac1, &tac2, 0, tac1.sampleNr-1);
  if(ret!=TPCERROR_OK) {tacFree(&tac1); tacFree(&tac2); return 21;}

  ret=tacCompareWeights(&tac1, &tac2, 1.0E-08, -1.0, NULL);
  if(ret!=0) {tacFree(&tac1); tacFree(&tac2); return 22;}
  
  tacFree(&tac1); tacFree(&tac2);
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_tacWMove(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }
  
  int ret;
  TAC tac; 
  tacInit(&tac);

  if(verbose>1) printf("\n trying with empty data\n");
  ret=tacWMove(&tac, 0, NULL);
  if(ret==TPCERROR_OK) {tacFree(&tac); return 1;}
  
  if(verbose>1) printf("\n creating test data\n");
  ret=create_tac(&tac); if(ret!=TPCERROR_OK) {tacFree(&tac); return 10;}

  if(verbose>1) printf("\n weight column not found \n");
  ret=tacWMove(&tac, 0, NULL);
  if(ret!=TPCERROR_NO_WEIGHTS) {tacFree(&tac); return 11;}

  if(verbose>1) printf("\n weight column is found; overwrite not allowed \n");
  strcpy(tac.c[0].name, "weight");
  tac.weighting=WEIGHTING_ON;
  ret=tacWMove(&tac, 0, NULL);
  if(ret!=TPCERROR_DUPLICATE_DATA) {tacFree(&tac); return 21;}

  if(verbose>1) printf("\n weight column is found; overwrite is allowed \n");
  tac.weighting=WEIGHTING_ON;
  ret=tacWMove(&tac, 1, NULL);
  if(ret!=TPCERROR_OK) {tacFree(&tac); return 22;}

  if(verbose>1) printf("\n weight column is found; no prev weights \n");
  tacFree(&tac);
  ret=create_tac(&tac); if(ret!=TPCERROR_OK) {tacFree(&tac); return 10;}
  strcpy(tac.c[0].name, "weight");
  tac.weighting=WEIGHTING_OFF;
  ret=tacWMove(&tac, 0, NULL);
  if(ret!=TPCERROR_OK) {tacFree(&tac); return 23;}

  tacFree(&tac);
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_tacWByFreq(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }
  
  int i, ret;
  TAC tac; tacInit(&tac);

  if(verbose>1) printf("\n trying with empty data\n");
  ret=tacWByFreq(&tac, NULL);
  if(ret==TPCERROR_OK) {tacFree(&tac); return 1;}
  
  if(verbose>1) printf("\n creating test data\n");
  ret=create_tac(&tac); if(ret!=TPCERROR_OK) {tacFree(&tac); return 10;}

  if(verbose>1) printf("\n trying with frames\n");
  tac.weighting=WEIGHTING_OFF;
  tac.isframe=1;
  ret=tacWByFreq(&tac, status);
  if(ret!=TPCERROR_OK) {tacFree(&tac); return 100;}
  if(tac.weighting!=WEIGHTING_ON) {tacFree(&tac); return 101;}
  for(i=0; i<tac.sampleNr; i++) {
    if(verbose>2) printf("  %g - %g : %g\n", tac.x1[i], tac.x2[i], tac.w[i]);
    if(!doubleMatch(tac.w[i], 1.0, 0.0001)) {tacFree(&tac); return 102;}
  }  

  if(verbose>1) printf("\n trying without frames\n");
  tac.weighting=WEIGHTING_OFF;
  tac.isframe=0;
  ret=tacWByFreq(&tac, status);
  if(ret!=TPCERROR_OK) {tacFree(&tac); return 200;}
  if(tac.weighting!=WEIGHTING_ON) {tacFree(&tac); return 201;}
  for(i=0; i<tac.sampleNr; i++) {
    if(verbose>2) printf("  %g : %g\n", tac.x[i], tac.w[i]);
    if(!doubleMatch(tac.w[i], 1.0, 0.0001)) {tacFree(&tac); return 202;}
  }  


  tacFree(&tac);
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_tacWSampleNr(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }
  
  
  TAC tac; tacInit(&tac);
  unsigned int n;
  int ret;

  if(verbose>1) printf("\n trying with empty data\n");
  n=tacWSampleNr(NULL);
  if(n>(unsigned int)0) {return 1;}
  n=tacWSampleNr(&tac);
  if(n>(unsigned int)0) {return 2;}
  
  if(verbose>1) printf("\n creating test data\n");
  ret=create_tac(&tac); if(ret!=TPCERROR_OK) {tacFree(&tac); return 10;}

  if(verbose>1) printf("\n trying with no weights\n");
  tac.weighting=WEIGHTING_OFF;
  n=tacWSampleNr(&tac);
  if(n!=(unsigned int)tac.sampleNr) {return 11;}

  if(verbose>1) printf("\n trying with weights\n");
  ret=tacWByFreq(&tac, status);
  if(ret!=TPCERROR_OK) {tacFree(&tac); return 100;}
  n=tacWSampleNr(&tac);
  if(n!=(unsigned int)tac.sampleNr) {return 21;}
  tac.w[1]=0.0;
  n=tacWSampleNr(&tac);
  if(n!=(unsigned int)tac.sampleNr-1) {return 22;}

  if(verbose>1) printf("\n trying with unknown weight status\n");
  tac.weighting=WEIGHTING_UNKNOWN;
  n=tacWSampleNr(&tac);
  if(n!=(unsigned int)tac.sampleNr-1) {return 31;}
  for(int i=1; i<tac.sampleNr; i++) tac.w[i]=nan("");
  n=tacWSampleNr(&tac); if(verbose>2) printf(" n=%u\n", n);
  if(n!=(unsigned int)1) {return 32;}
  for(int i=0; i<tac.sampleNr; i++) tac.w[i]=nan("");
  n=tacWSampleNr(&tac); if(verbose>2) printf(" n=%u\n", n);
  if(n!=(unsigned int)tac.sampleNr) {return 33;}


  tacFree(&tac);
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
