#!/bin/bash
#: Title      : test_tacstudy
#: Date       : 2014-09-25
#: Author     : "Vesa Oikonen" <vesa.oikonen@utu.fi>
#: Options    : None

# Set the name of executable to test
if [ -n "${OS+1}" ] && [ "$OS" = "Windows_NT" ]; then
EXT=.exe;
else
EXT=;
fi
PROGRAM=../../tacstudy$EXT;

printf "=====================================================================\n"
printf "creating test data for  %s\n" $PROGRAM
printf "=====================================================================\n"

printf "\n verifying that required data exists \n"

if [ ! -f $PROGRAM ]; then
  printf "Failed: executable does not exist.\n"
  exit 1
fi

if [ ! -f data.dft ]; then
  printf "Failed: required file does not exist.\n"
  exit 1
fi



printf "\n\n"
printf "=====================================================================\n"
printf "testing %s\n" $PROGRAM
printf "=====================================================================\n"



printf "\n 1.0.0 No option, just print study number, which does not exist \n\n"

rm -f stdout.txt test.dft
cp -f data.dft test.dft

$PROGRAM test.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "test.dft"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 1.1.0 set study name with valid command line argument \n\n"

rm -f stdout.txt test.dft
cp -f data.dft test.dft

$PROGRAM test.dft uai546
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM test.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "test.dft" "uai546"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 1.2.0 set study name with filename with path as command line argument \n\n"

rm -f stdout.txt test.dft
cp -f data.dft test.dft

$PROGRAM test.dft /h/mankeli/uai546dy1.v
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM test.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "test.dft" "uai546"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 1.2.0 set study name with filename without path as command line argument \n\n"

rm -f stdout.txt test.dft
cp -f data.dft test.dft

$PROGRAM test.dft uai546dy1.v
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM test.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "test.dft" "uai546"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 2.0.0 set study name from name of datafile \n\n"

rm -f stdout.txt test66.dft
cp -f data.dft test66.dft

$PROGRAM -fn test66.dft
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM test66.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "test66.dft" "test66"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"
# New studynumber could just be from filename, so change the name
cp -f test66.dft temp66.dft
$PROGRAM temp66.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "temp66.dft" "test66"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 2.1.0 set study name from 5 first letters of name of datafile \n\n"

rm -f stdout.txt test66.dft
cp -f data.dft test66.dft

$PROGRAM -fn=5 test66.dft
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM test66.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "test66.dft" "test6"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"
# New studynumber could just be from filename, so change the name
cp -f test66.dft temp66.dft
$PROGRAM temp66.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "temp66.dft" "test6"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"



printf "\n 2.2.0 path should not matter \n\n"

rm -f stdout.txt test66.dft
cp -f data.dft test66.dft

$PROGRAM -fn ./test66.dft
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM test66.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "test66.dft" "test66"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"
# New studynumber could just be from filename, so change the name
cp -f test66.dft temp66.dft
$PROGRAM temp66.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "temp66.dft" "test66"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 2.3.0 not even with 5 first letters \n\n"

rm -f stdout.txt test66.dft
cp -f data.dft test66.dft

$PROGRAM -fn=5 ./test66.dft
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM test66.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "test66.dft" "test6"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"
# New studynumber could just be from filename, so change the name
rm -f stdout.txt
cp -f test66.dft temp66.dft
$PROGRAM temp66.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "temp66.dft" "test6"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 2.4.0 fails if filename is not valid study number \n\n"

rm -f stdout.txt test.dft
cp -f data.dft test.dft

$PROGRAM -fn ./test.dft &> stdout.txt
if [ $? -eq 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt Error "cannot get study number from filename."
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 2.5.0 can be forced if filename is not valid study number \n\n"

rm -f stdout.txt test.dft
cp -f data.dft test.dft

$PROGRAM -fn -force ./test.dft
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM test.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "test.dft" "test"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"
# New studynumber could just be from filename, so change the name
rm -f stdout.txt
cp -f test.dft temp66.dft
$PROGRAM temp66.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "temp66.dft" "test"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"



printf "\n 3.0.0 delete first letters from the argument study number \n\n"

rm -f stdout.txt test.dft
cp -f data.dft test.dft

$PROGRAM -df=4 ./test.dft testi6789
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM test.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "test.dft" "i6789"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 3.1.0 delete first letters from filename \n\n"

rm -f stdout.txt testi6789.dft
cp -f data.dft testi6789.dft

$PROGRAM -df=4 -fn ./testi6789.dft
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM testi6789.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "testi6789.dft" "i6789"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 3.2.0 delete first and end letters from filename \n\n"

rm -f stdout.txt testi123456789.dft
cp -f data.dft testi123456789.dft

$PROGRAM -df=4 -fn=6 ./testi123456789.dft
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM testi123456789.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "testi123456789.dft" "i12345"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 3.3.0 fails if too many numbers left \n\n"

rm -f stdout.txt testi123456789.dft
cp -f data.dft testi123456789.dft

$PROGRAM -df=4 -fn=7 ./testi123456789.dft &> stdout.txt
if [ $? -eq 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt Error "invalid study number."
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"




printf "\n 4.0.0 valid study number is found inside filename \n\n"

rm -f stdout.txt patientname_test0066_vers2.dft
cp -f data.dft patientname_test0066_vers2.dft

$PROGRAM -fn ./patientname_test0066_vers2.dft
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM patientname_test0066_vers2.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "patientname_test0066_vers2.dft" "test66"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"
# New studynumber could just be from filename, so change the name
cp -f patientname_test0066_vers2.dft temp88.dft
$PROGRAM temp88.dft &> stdout.txt
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "temp88.dft" "test66"
if [ $? -ne 0 ] ; then printf "\nFailed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"



printf "=====================================================================\n"
printf "  All passed!\n"
printf "=====================================================================\n"
exit 0
